#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
"""Provide handling of a location."""
from .gateway import Gateway


class Location(object):
    """Provide handling of a location."""

    def __init__(self, client, data=None):
        """Initialise the class."""
        self.client = client
        self._gateways = []
        self.gateways = {}
        self.locationId = None

        if data is not None:
            self.__dict__.update(data["locationInfo"])

            for gw_data in data["gateways"]:
                gateway = Gateway(client, self, gw_data)
                self._gateways.append(gateway)
                self.gateways[gateway.gatewayId] = gateway

    async def status(self):
        """Retrieve the location status."""
        url = (
            "https://tccna.honeywell.com/WebAPI/emea/api/v1/"
            "location/%s/status?includeTemperatureControlSystems=True" % self.locationId
        )

        async with self.client._session.get(
            url,
            headers=await self.client._headers(),
        ) as response:
            response.raise_for_status()
            data = await response.json()

        # Now feed into other elements
        for gw_data in data["gateways"]:
            gateway = self.gateways[gw_data["gatewayId"]]

            for sys in gw_data["temperatureControlSystems"]:
                system = gateway.control_systems[sys["systemId"]]

                system.__dict__.update(
                    {
                        "systemModeStatus": sys["systemModeStatus"],
                        "activeFaults": sys["activeFaults"],
                    }
                )

                if "dhw" in sys:
                    system.hotwater.__dict__.update(sys["dhw"])

                for zone_data in sys["zones"]:
                    zone = system.zones[zone_data["name"]]
                    zone.__dict__.update(zone_data)

        return data



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
"""Provide handling of the hot water zone."""
from .zone import ZoneBase


class HotWater(ZoneBase):
    """Provide handling of the hot water zone."""

    def __init__(self, client, data):
        """Initialise the class."""
        super(HotWater, self).__init__(client)

        self.dhwId = None

        self.__dict__.update(data)

        self.name = ""
        self.zoneId = self.dhwId
        self.zone_type = "domesticHotWater"

    async def _set_dhw(self, data):  # TODO: deprecate
        return await self._set_dhw_state(data)

    async def _set_dhw_state(self, data):
        """Get the DHW state."""
        headers = dict(await self.client._headers())
        headers["Content-Type"] = "application/json"

        url = (
            "https://tccna.honeywell.com/WebAPI/emea/api/v1/domesticHotWater/%s/state"
            % self.dhwId
        )

        async with self.client._session.put(
            url, json=data, headers=headers
        ) as response:
            response.raise_for_status()

    async def set_dhw_on(self, until=None):
        """Set the DHW on until a given time, or permanently."""
        if until is None:
            data = {"Mode": "PermanentOverride", "State": "On", "UntilTime": None}
        else:
            data = {
                "Mode": "TemporaryOverride",
                "State": "On",
                "UntilTime": until.strftime("%Y-%m-%dT%H:%M:%SZ"),
            }

        await self._set_dhw_state(data)

    async def set_dhw_off(self, until=None):
        """Set the DHW off until a given time, or permanently."""
        if until is None:
            data = {"Mode": "PermanentOverride", "State": "Off", "UntilTime": None}
        else:
            data = {
                "Mode": "TemporaryOverride",
                "State": "Off",
                "UntilTime": until.strftime("%Y-%m-%dT%H:%M:%SZ"),
            }

        await self._set_dhw_state(data)

    async def set_dhw_auto(self):
        """Set the DHW to follow the schedule."""
        data = {"Mode": "FollowSchedule", "State": "", "UntilTime": None}

        await self._set_dhw_state(data)

    async def get_dhw_state(self):
        """Get the DHW state."""
        headers = dict(await self.client._headers())
        headers["Content-Type"] = "application/json"

        url = (
            "https://tccna.honeywell.com/WebAPI/emea/api/v1/domesticHotWater/%s/status?"
            % self.dhwId
        )

        async with self.client._session.get(url, headers=headers) as response:
            response.raise_for_status()
            data = await response.json()
        return data



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
"""Provides handling of a control system."""
import json
import logging

from .hotwater import HotWater
from .zone import Zone

_LOGGER = logging.getLogger(__name__)


class ControlSystem(object):
    """Provides handling of a control system."""

    def __init__(self, client, location, gateway, data=None):
        """Initialise the class."""
        self.client = client
        self.location = location
        self.gateway = gateway

        self._zones = []
        self.zones = {}
        self.zones_by_id = {}
        self.hotwater = None
        self.systemId = None

        if data is not None:
            local_data = dict(data)
            del local_data["zones"]
            self.__dict__.update(local_data)

            for z_data in data["zones"]:
                zone = Zone(client, z_data)
                self._zones.append(zone)
                self.zones[zone.name] = zone
                self.zones_by_id[zone.zoneId] = zone

            if "dhw" in data:
                self.hotwater = HotWater(client, data["dhw"])

    async def _set_status(self, mode, until=None):
        headers = dict(await self.client._headers())
        headers["Content-Type"] = "application/json"

        if until is None:
            data = {"SystemMode": mode, "TimeUntil": None, "Permanent": True}
        else:
            data = {
                "SystemMode": mode,
                "TimeUntil": until.strftime("%Y-%m-%dT%H:%M:%SZ"),
                "Permanent": False,
            }

        url = (
            "https://tccna.honeywell.com/WebAPI/emea/api/v1"
            "/temperatureControlSystem/%s/mode" % self.systemId
        )

        async with self.client._session.put(
            url, json=data, headers=await self.client._headers()
        ) as response:
            response.raise_for_status()

    async def set_status(self, mode, until=None):
        """Set the system to a mode, either indefinitely, or for a set time."""
        await self._set_status(mode, until)

    async def set_status_normal(self):
        """Set the system into normal mode."""
        await self._set_status("Auto")

    async def set_status_reset(self):
        """Reset the system into normal mode.

        This will also set all the zones to FollowSchedule mode.
        """
        await self._set_status("AutoWithReset")

    async def set_status_custom(self, until=None):
        """Set the system into custom mode."""
        await self._set_status("Custom", until)

    async def set_status_eco(self, until=None):
        """Set the system into eco mode."""
        await self._set_status("AutoWithEco", until)

    async def set_status_away(self, until=None):
        """Set the system into away mode."""
        await self._set_status("Away", until)

    async def set_status_dayoff(self, until=None):
        """Set the system into dayoff mode."""
        await self._set_status("DayOff", until)

    async def set_status_heatingoff(self, until=None):
        """Set the system into heating off mode."""
        await self._set_status("HeatingOff", until)

    async def temperatures(self):
        """Return a generator with the details of each zone."""
        await self.location.status()

        if self.hotwater:
            yield {
                "thermostat": "DOMESTIC_HOT_WATER",
                "id": self.hotwater.dhwId,
                "name": "",
                "temp": self.hotwater.temperatureStatus["temperature"],
                "setpoint": "",
            }

        for zone in self._zones:
            zone_info = {
                "thermostat": "EMEA_ZONE",
                "id": zone.zoneId,
                "name": zone.name,
                "temp": None,
                "setpoint": zone.setpointStatus["targetHeatTemperature"],
            }

            if zone.temperatureStatus["isAvailable"]:
                zone_info["temp"] = zone.temperatureStatus["temperature"]
            yield zone_info

    def zone_schedules_backup(self, filename):
        """Backup all zones on control system to the given file."""
        _LOGGER.info(
            "Backing up schedules from ControlSystem: %s (%s)...",
            self.systemId,
            self.location.name,
        )

        schedules = {}

        if self.hotwater:
            _LOGGER.info("Retrieving DHW schedule: %s...", self.hotwater.zoneId)

            schedule = self.hotwater.schedule()
            schedules[self.hotwater.zoneId] = {
                "name": "Domestic Hot Water",
                "schedule": schedule,
            }

        for zone in self._zones:
            zone_id = zone.zoneId
            name = zone.name

            _LOGGER.info("Retrieving Zone schedule: %s - %s", zone_id, name)

            schedule = zone.schedule()
            schedules[zone_id] = {"name": name, "schedule": schedule}

        schedule_db = json.dumps(schedules, indent=4)

        _LOGGER.info("Writing to backup file: %s...", filename)
        with open(filename, "w") as file_output:
            file_output.write(schedule_db)

        _LOGGER.info("Backup completed.")

    def zone_schedules_restore(self, filename):
        """Restore all zones on control system from the given file."""
        _LOGGER.info(
            "Restoring schedules to ControlSystem %s (%s)...",
            self.systemId,
            self.location,
        )

        _LOGGER.info("Reading from backup file: %s...", filename)
        with open(filename, "r") as file_input:
            schedule_db = file_input.read()
            schedules = json.loads(schedule_db)

            for zone_id, zone_schedule in schedules.items():
                name = zone_schedule["name"]
                zone_info = zone_schedule["schedule"]

                _LOGGER.info("Restoring schedule for: %s - %s...", zone_id, name)

                if self.hotwater and self.hotwater.zoneId == zone_id:
                    self.hotwater.set_schedule(json.dumps(zone_info))
                else:
                    self.zones_by_id[zone_id].set_schedule(json.dumps(zone_info))

        _LOGGER.info("Restore completed.")



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
"""evohomeasync provides an async client for the oiginal Evohome API.

It is a faithful async port of https://github.com/watchforstock/evohome-client

Further information at: https://evohome-client.readthedocs.io
"""
import json
import logging
import time

import aiohttp

HTTP_UNAUTHORIZED = 401
HTTP_OK = 200

logging.basicConfig()
_LOGGER = logging.getLogger(__name__)


class EvohomeClient(object):
    """Provide a client to access the Honeywell Evohome system."""

    def __init__(self, username, password, user_data=None, **kwargs):
        """Take the username and password for the service.

        If user_data is given then this will be used to try and reduce the number of
        calls to the authentication service which is known to be rate limited.
        """
        if kwargs.get("debug") is True:  # if debug is True:
            _LOGGER.setLevel(logging.DEBUG)
            _LOGGER.debug("Debug mode is explicitly enabled.")
        else:
            _LOGGER.debug(
                "Debug mode is not explicitly enabled (but may be enabled elsewhere)."
            )

        self.username = username
        self.password = password

        self.user_data = user_data
        self.hostname = kwargs.get("hostname", "https://tccna.honeywell.com")

        self._session = kwargs.get("session") or aiohttp.ClientSession(
            timeout=aiohttp.ClientTimeout(total=30)
        )

        self.full_data = None
        self.gateway_data = None

        self.location_id = ""
        self.devices = {}
        self.named_devices = {}
        self.postdata = {}
        self.headers = {}

    async def _populate_full_data(self, force_refresh=False):
        if self.full_data is None or force_refresh:
            await self._populate_user_info()

            user_id = self.user_data["userInfo"]["userID"]
            session_id = self.user_data["sessionId"]

            url = (
                self.hostname + "/WebAPI/api/locations?userId=%s&allData=True" % user_id
            )
            self.headers["sessionId"] = session_id

            response = await self._do_request("get", url, json.dumps(self.postdata))

            self.full_data = list(await response.json())[0]

            self.location_id = self.full_data["locationID"]

            self.devices = {}
            self.named_devices = {}

            for device in self.full_data["devices"]:
                self.devices[device["deviceID"]] = device
                self.named_devices[device["name"]] = device

    async def _populate_user_info(self):
        if self.user_data is None:
            url = self.hostname + "/WebAPI/api/Session"
            self.postdata = {
                "Username": self.username,
                "Password": self.password,
                "ApplicationId": "91db1612-73fd-4500-91b2-e63b069b185c",
            }
            self.headers = {"content-type": "application/json"}

            response = await self._do_request(
                "post", url, data=json.dumps(self.postdata), retry=False
            )

            self.user_data = await response.json()

        return self.user_data

    async def temperatures(self, force_refresh=False):
        """Retrieve the current details for each zone."""
        await self._populate_full_data(force_refresh)

        result = []
        for device in self.full_data["devices"]:
            set_point = 0
            status = ""
            if "heatSetpoint" in device["thermostat"]["changeableValues"]:
                set_point = float(
                    device["thermostat"]["changeableValues"]["heatSetpoint"]["value"]
                )
                status = device["thermostat"]["changeableValues"]["heatSetpoint"][
                    "status"
                ]

            else:
                status = device["thermostat"]["changeableValues"]["status"]
            result.append(
                {
                    "thermostat": device["thermostatModelType"],
                    "id": device["deviceID"],
                    "name": device["name"],
                    "temp": float(device["thermostat"]["indoorTemperature"]),
                    "setpoint": set_point,
                    "status": status,
                    "mode": device["thermostat"]["changeableValues"]["mode"],
                }
            )

        return result

    async def get_modes(self, zone):
        """Return the set of modes the device can be assigned."""
        await self._populate_full_data()
        device = self._get_device(zone)
        return device["thermostat"]["allowedModes"]

    def _get_device(self, zone):
        if isinstance(zone, str):
            return self.named_devices[zone]
        return self.devices[zone]

    async def _get_task_status(self, task_id):
        await self._populate_full_data()
        url = self.hostname + "/WebAPI/api/commTasks?commTaskId=%s" % task_id

        response = await self._do_request("get", url)

        return dict(await response.json())["state"]

    async def _get_task_id(self, response):
        ret = await response.json()
        return ret[0]["id"] if isinstance(ret, list) else ret["id"]

    async def _do_request(self, method, url, data=None, retry=True):
        if method == "get":
            func = self._session.get
        elif method == "put":
            func = self._session.put
        elif method == "post":
            func = self._session.post

        async with func(url, data=data, headers=self.headers) as response:
            response_text = await response.text()

            # catch 401/unauthorized since we may retry
            if response.status == HTTP_UNAUTHORIZED and retry is True:
                # Attempt to refresh sessionId if it has expired
                if "code" in response_text:  # don't use response.json() here!
                    response_json = await response.json()
                    if response_json[0]["code"] == "Unauthorized":
                        _LOGGER.debug("Session expired, re-authenticating...")
                        # Get a new sessionId
                        self.user_data = None
                        await self._populate_user_info()
                        # Set headers with new sessionId
                        session_id = self.user_data["sessionId"]
                        self.headers["sessionId"] = session_id
                        _LOGGER.debug("sessionId = %s", session_id)

                        response = await self._do_request(
                            method, url, data=data, retry=False
                        )

            # display error message if the vendor provided one
            if response.status != HTTP_OK:
                if "code" in response_text:  # don't use response.json()!
                    _LOGGER.error(
                        "HTTP Status = %s, Response = %s",
                        response.status,
                        response_text,
                    )

            response.raise_for_status()

        return response

    async def _set_status(self, status, until=None):
        await self._populate_full_data()
        url = (
            self.hostname
            + "/WebAPI/api/evoTouchSystems?locationId=%s" % self.location_id
        )

        if until is None:
            data = {"QuickAction": status, "QuickActionNextTime": None}
        else:
            data = {
                "QuickAction": status,
                "QuickActionNextTime": until.strftime("%Y-%m-%dT%H:%M:%SZ"),
            }

        response = await self._do_request("put", url, json.dumps(data))

        task_id = await self._get_task_id(response)

        while await self._get_task_status(task_id) != "Succeeded":
            time.sleep(1)

    async def set_status_normal(self):
        """Set the system to normal operation."""
        await self._set_status("Auto")

    async def set_status_custom(self, until=None):
        """Set the system to the custom programme."""
        await self._set_status("Custom", until)

    async def set_status_eco(self, until=None):
        """Set the system to the eco mode."""
        await self._set_status("AutoWithEco", until)

    async def set_status_away(self, until=None):
        """Set the system to the away mode."""
        await self._set_status("Away", until)

    async def set_status_dayoff(self, until=None):
        """Set the system to the day off mode."""
        await self._set_status("DayOff", until)

    async def set_status_heatingoff(self, until=None):
        """Set the system to the heating off mode."""
        await self._set_status("HeatingOff", until)

    def _get_device_id(self, zone):
        device = self._get_device(zone)
        return device["deviceID"]

    async def _set_heat_setpoint(self, zone, data):
        await self._populate_full_data()

        device_id = self._get_device_id(zone)

        url = (
            self.hostname
            + "/WebAPI/api/devices/%s/thermostat/changeableValues/heatSetpoint"
            % device_id
        )

        response = await self._do_request("put", url, json.dumps(data))

        task_id = await self._get_task_id(response)

        while await self._get_task_status(task_id) != "Succeeded":
            time.sleep(1)

    async def set_temperature(self, zone, temperature, until=None):
        """Set the temperature of the given zone."""
        if until is None:
            data = {"Value": temperature, "Status": "Hold", "NextTime": None}
        else:
            data = {
                "Value": temperature,
                "Status": "Temporary",
                "NextTime": until.strftime("%Y-%m-%dT%H:%M:%SZ"),
            }

        await self._set_heat_setpoint(zone, data)

    async def cancel_temp_override(self, zone):
        """Remove an existing temperature override."""
        data = {"Value": None, "Status": "Scheduled", "NextTime": None}
        await self._set_heat_setpoint(zone, data)

    def _get_dhw_zone(self):
        for device in self.full_data["devices"]:
            if device["thermostatModelType"] == "DOMESTIC_HOT_WATER":
                return device["deviceID"]

    async def _set_dhw(self, status="Scheduled", mode=None, next_time=None):
        """Set DHW to On, Off or Auto, either indefinitely, or until a set time."""
        data = {
            "Status": status,
            "Mode": mode,
            "NextTime": next_time,
            "SpecialModes": None,
            "HeatSetpoint": None,
            "CoolSetpoint": None,
        }

        await self._populate_full_data()
        dhw_zone = self._get_dhw_zone()
        if dhw_zone is None:
            raise Exception("No DHW zone reported from API")
        url = (
            self.hostname
            + "/WebAPI/api/devices/%s/thermostat/changeableValues" % dhw_zone
        )

        response = await self._do_request("put", url, json.dumps(data))

        task_id = await self._get_task_id(response)

        while await self._get_task_status(task_id) != "Succeeded":
            time.sleep(1)

    async def set_dhw_on(self, until=None):
        """Set DHW to on, either indefinitely, or until a specified time.

        When On, the DHW controller will work to keep its target temperature at/above
        its target temperature.  After the specified time, it will revert to its
        scheduled behaviour.
        """
        time_until = None if until is None else until.strftime("%Y-%m-%dT%H:%M:%SZ")

        await self._set_dhw(status="Hold", mode="DHWOn", next_time=time_until)

    async def set_dhw_off(self, until=None):
        """Set DHW to on, either indefinitely, or until a specified time.

        When Off, the DHW controller will ignore its target temperature. After the
        specified time, it will revert to its scheduled behaviour.
        """
        time_until = None if until is None else until.strftime("%Y-%m-%dT%H:%M:%SZ")

        await self._set_dhw(status="Hold", mode="DHWOff", next_time=time_until)

    async def set_dhw_auto(self):
        """Set DHW to On or Off, according to its schedule."""
        await self._set_dhw(status="Scheduled")



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
"""evohomeasync2 provides an async client for the updated Evohome API.

It is a faithful async port of https://github.com/watchforstock/evohome-client

Further information at: https://evohome-client.readthedocs.io
"""
import logging
from datetime import datetime, timedelta

import aiohttp

from .location import Location

HTTP_UNAUTHORIZED = 401

logging.basicConfig()
_LOGGER = logging.getLogger(__name__)

HEADER_ACCEPT = (
    "application/json, application/xml, text/json, text/x-json, "
    "text/javascript, text/xml"
)
HEADER_AUTHORIZATION_BASIC = (
    "Basic "
    "NGEyMzEwODktZDJiNi00MWJkLWE1ZWItMTZhMGE0MjJiOTk5OjFhMTVjZGI4LTQyZGUtNDA3Y"
    "i1hZGQwLTA1OWY5MmM1MzBjYg=="
)
HEADER_BASIC_AUTH = {
    "Accept": HEADER_ACCEPT,
    "Authorization": HEADER_AUTHORIZATION_BASIC,
}


class AuthenticationError(Exception):
    """Exception raised when unable to get an access_token."""

    def __init__(self, message):
        """Construct the AuthenticationError object."""
        self.message = message
        super(AuthenticationError, self).__init__(message)


class EvohomeClient(object):
    """Provide access to the v2 Evohome API."""

    def __init__(self, username, password, **kwargs):
        """Construct the EvohomeClient object."""
        if kwargs.get("debug") is True:
            _LOGGER.setLevel(logging.DEBUG)
            _LOGGER.debug("Debug mode is explicitly enabled.")
        else:
            _LOGGER.debug(
                "Debug mode is not explicitly enabled (but may be enabled elsewhere)."
            )

        self.username = username
        self.password = password

        self.refresh_token = kwargs.get("refresh_token")
        self.access_token = kwargs.get("access_token")
        self.access_token_expires = kwargs.get("access_token_expires")

        self._session = kwargs.get("session") or aiohttp.ClientSession(
            timeout=aiohttp.ClientTimeout(total=30)
        )

        self.account_info = None
        self.locations = None
        self.installation_info = None
        self.system_id = None

    async def login(self):
        """Authenticate with the server."""
        try:  # the cached access_token may be valid, but is not authorized
            await self.user_account()
        except aiohttp.ClientResponseError as err:
            if err.status != HTTP_UNAUTHORIZED or not self.access_token:
                raise

            _LOGGER.warning("Unauthorized access_token (will try re-authenticating).")
            self.access_token = None
            await self.user_account()

        await self.installation()

    async def _headers(self):
        """Ensure the Authorization Header has a valid Access Token."""
        if not self.access_token or not self.access_token_expires:
            await self._basic_login()

        elif datetime.now() > self.access_token_expires - timedelta(seconds=30):
            await self._basic_login()

        return {"Accept": HEADER_ACCEPT, "Authorization": "bearer " + self.access_token}

    async def _basic_login(self):
        """Obtain a new access token from the vendor.

        First, try using the refresh_token, if one is available, otherwise
        authenticate using the user credentials.
        """
        _LOGGER.debug("No/Expired/Invalid access_token, re-authenticating.")
        self.access_token = self.access_token_expires = None

        if self.refresh_token:
            _LOGGER.debug("Authenticating with the refresh_token...")
            credentials = {
                "grant_type": "refresh_token",
                "scope": "EMEA-V1-Basic EMEA-V1-Anonymous",
                "refresh_token": self.refresh_token,
            }

            try:
                await self._obtain_access_token(credentials)
            except AuthenticationError:
                _LOGGER.warning("Invalid refresh_token (will try user credentials).")
                self.refresh_token = None

        if not self.refresh_token:
            _LOGGER.debug("Authenticating with the user credentials...")
            credentials = {
                "grant_type": "password",
                "scope": "EMEA-V1-Basic EMEA-V1-Anonymous "
                "EMEA-V1-Get-Current-User-Account",
                "Username": self.username,
                "Password": self.password,
            }

            await self._obtain_access_token(credentials)

        _LOGGER.debug("refresh_token = %s", self.refresh_token)
        _LOGGER.debug("access_token = %s", self.access_token)
        _LOGGER.debug(
            "access_token_expires = %s",
            self.access_token_expires.strftime("%Y-%m-%d %H:%M:%S"),
        )

    async def _obtain_access_token(self, credentials):
        url = "https://tccna.honeywell.com/Auth/OAuth/Token"
        payload = {
            "Content-Type": "application/x-www-form-urlencoded; charset=utf-8",
            "Host": "rs.alarmnet.com/",
            "Cache-Control": "no-store no-cache",
            "Pragma": "no-cache",
            "Connection": "Keep-Alive",
        }
        payload.update(credentials)  # merge the credentials into the payload

        async with self._session.post(
            url,
            data=payload,
            headers=HEADER_BASIC_AUTH,
        ) as response:
            try:
                response_text = await response.text()
                response.raise_for_status()

            except aiohttp.ClientResponseError:
                msg = "Unable to obtain an Access Token"
                if response_text:  # if there is a message, then raise with it
                    msg = msg + ", hint: " + response_text

                raise AuthenticationError(msg)

            try:  # the access token _should_ be valid...
                # this may cause a ValueError
                response_json = await response.json()

                # these may cause a KeyError
                self.access_token = response_json["access_token"]
                self.access_token_expires = datetime.now() + timedelta(
                    seconds=response_json["expires_in"]
                )
                self.refresh_token = response_json["refresh_token"]

            except KeyError:
                raise AuthenticationError(
                    "Unable to obtain an Access Token, hint: " + response_json
                )

            except ValueError:
                raise AuthenticationError(
                    "Unable to obtain an Access Token, hint: " + response_text
                )

    def _get_location(self, location):
        if location is None:
            return self.installation_info[0]["locationInfo"]["locationId"]
        return location

    def _get_single_heating_system(self):
        # This allows a shortcut for some systems
        if len(self.locations) != 1:
            raise Exception("More (or less) than one location available")

        if len(self.locations[0]._gateways) != 1:
            raise Exception("More (or less) than one gateway available")

        if len(self.locations[0]._gateways[0]._control_systems) != 1:
            raise Exception("More (or less) than one control system available")

        return self.locations[0]._gateways[0]._control_systems[0]

    async def user_account(self):
        """Return the user account information."""
        self.account_info = None

        url = "https://tccna.honeywell.com/WebAPI/emea/api/v1/userAccount"

        async with self._session.get(url, headers=await self._headers()) as response:
            response.raise_for_status()

            self.account_info = await response.json()
            return self.account_info

    async def installation(self):
        """Return the details of the installation."""
        self.locations = []

        url = (
            "https://tccna.honeywell.com/WebAPI/emea/api/v1/location"
            "/installationInfo?userId=%s&includeTemperatureControlSystems=True"
            % self.account_info["userId"]
        )

        async with self._session.get(url, headers=await self._headers()) as response:
            response.raise_for_status()

            self.installation_info = await response.json()

        self.system_id = self.installation_info[0]["gateways"][0][
            "temperatureControlSystems"
        ][0]["systemId"]

        for loc_data in self.installation_info:
            location = Location(self, loc_data)
            await location.status()
            self.locations.append(location)

        return self.installation_info

    async def full_installation(self, location=None):
        """Return the full details of the installation."""
        url = (
            "https://tccna.honeywell.com/WebAPI/emea/api/v1/location"
            "/%s/installationInfo?includeTemperatureControlSystems=True"
            % self._get_location(location)
        )

        async with self._session.get(url, headers=await self._headers()) as response:
            response.raise_for_status()

            return await response.json()

    async def gateway(self):
        """Return the details of the gateway."""
        url = "https://tccna.honeywell.com/WebAPI/emea/api/v1/gateway"

        async with self._session.get(url, headers=await self._headers()) as response:
            response.raise_for_status()

            return await response.json()

    async def set_status_normal(self):
        """Set the system into normal heating mode."""
        return await self._get_single_heating_system().set_status_normal()

    async def set_status_reset(self):
        """Reset the system mode."""
        return await self._get_single_heating_system().set_status_reset()

    async def set_status_custom(self, until=None):
        """Set the system into custom heating mode."""
        return await self._get_single_heating_system().set_status_custom(until)

    async def set_status_eco(self, until=None):
        """Set the system into eco heating mode."""
        return await self._get_single_heating_system().set_status_eco(until)

    async def set_status_away(self, until=None):
        """Set the system into away heating mode."""
        return await self._get_single_heating_system().set_status_away(until)

    async def set_status_dayoff(self, until=None):
        """Set the system into day off heating mode."""
        return await self._get_single_heating_system().set_status_dayoff(until)

    async def set_status_heatingoff(self, until=None):
        """Set the system into heating off heating mode."""
        return await self._get_single_heating_system().set_status_heatingoff(until)

    async def temperatures(self):
        """Return the current zone temperatures and set points."""
        return await self._get_single_heating_system().temperatures()

    def zone_schedules_backup(self, filename):
        """Back up the current system configuration to the given file."""
        return self._get_single_heating_system().zone_schedules_backup(filename)

    def zone_schedules_restore(self, filename):
        """Restore the current system configuration from the given file."""
        return self._get_single_heating_system().zone_schedules_restore(filename)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
"""Provide handling of individual zones."""
import json


class ZoneBase(object):
    """Provide the base for Zones."""

    def __init__(self, client):
        """Initialise the class."""
        self.client = client
        self.name = None
        self.zoneId = None
        self.zone_type = None

    async def schedule(self):
        """Get the schedule for the given zone."""
        url = "https://tccna.honeywell.com/WebAPI/emea/api/v1/%s/%s/schedule" % (
            self.zone_type,
            self.zoneId,
        )

        async with self.client._session.get(
            url, headers=await self.client._headers()
        ) as response:
            response.raise_for_status()
            response_data = await response.text()

        mapping = [
            ("dailySchedules", "DailySchedules"),
            ("dayOfWeek", "DayOfWeek"),
            ("temperature", "TargetTemperature"),
            ("timeOfDay", "TimeOfDay"),
            ("switchpoints", "Switchpoints"),
            ("dhwState", "DhwState"),
        ]

        for from_val, to_val in mapping:
            response_data = response_data.replace(from_val, to_val)

        data = json.loads(response_data)
        # change the day name string to a number offset (0 = Monday)
        for day_of_week, schedule in enumerate(data["DailySchedules"]):
            schedule["DayOfWeek"] = day_of_week
        return data

    async def set_schedule(self, zone_info):
        """Set the schedule for this zone."""
        # must only POST json, otherwise server API handler raises exceptions
        try:
            json.loads(zone_info)

        except ValueError as error:
            raise ValueError("zone_info must be valid JSON: ", error)

        headers = dict(await self.client._headers())
        headers["Content-Type"] = "application/json"

        url = "https://tccna.honeywell.com/WebAPI/emea/api/v1/%s/%s/schedule" % (
            self.zone_type,
            self.zoneId,
        )

        async with self.client._session.put(
            url, data=zone_info, headers=headers
        ) as response:
            response.raise_for_status()

            return await response.json()


class Zone(ZoneBase):
    """Provide the access to an individual zone."""

    def __init__(self, client, data):
        """Initialise the class."""
        super(Zone, self).__init__(client)

        self.__dict__.update(data)

        self.zone_type = "temperatureZone"

    async def set_temperature(self, temperature, until=None):
        """Set the temperature of the given zone."""
        if until is None:
            data = {
                "SetpointMode": "PermanentOverride",
                "HeatSetpointValue": temperature,
                "TimeUntil": None,
            }
        else:
            data = {
                "SetpointMode": "TemporaryOverride",
                "HeatSetpointValue": temperature,
                "TimeUntil": until.strftime("%Y-%m-%dT%H:%M:%SZ"),
            }

        await self._set_heat_setpoint(data)

    async def _set_heat_setpoint(self, data):
        headers = dict(await self.client._headers())
        headers["Content-Type"] = "application/json"

        url = (
            "https://tccna.honeywell.com/WebAPI/emea/api/v1"
            "/temperatureZone/%s/heatSetpoint" % self.zoneId
        )

        async with self.client._session.put(
            url, json=data, headers=headers
        ) as response:
            response.raise_for_status()

    async def cancel_temp_override(self):
        """Cancel an override to the zone temperature."""
        data = {
            "SetpointMode": "FollowSchedule",
            "HeatSetpointValue": 0.0,
            "TimeUntil": None,
        }

        await self._set_heat_setpoint(data)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
"""Provide handling of a gateway."""
from .controlsystem import ControlSystem


class Gateway(object):
    """Provide handling of a gateway."""

    def __init__(self, client, location, data=None):
        """Initialise the class."""
        self.client = client
        self.location = location
        self._control_systems = []
        self.control_systems = {}

        if data is not None:
            self.__dict__.update(data["gatewayInfo"])

            for cs_data in data["temperatureControlSystems"]:
                control_system = ControlSystem(client, location, self, cs_data)
                self._control_systems.append(control_system)
                self.control_systems[control_system.systemId] = control_system



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
"""The setup.py file."""

import os
import sys

from setuptools import find_packages, setup
from setuptools.command.install import install

VERSION = "0.3.15"

URL = "https://github.com/zxdavb/evohome-async"

with open("README.md", "r") as fh:
    LONG_DESCRIPTION = fh.read()


class VerifyVersionCommand(install):
    """Custom command to verify that the git tag matches our VERSION."""

    def run(self):
        tag = os.getenv("CIRCLE_TAG")
        if tag != VERSION:
            info = f"The git tag: '{tag}' does not match the package ver: '{VERSION}'"
            sys.exit(info)


setup(
    name="evohome-async",
    description="An async client for connecting to Honeywell's TCC RESTful API.",
    keywords=["evohome", "total connect comfort", "round thermostat"],
    author="Andrew Stock & David Bonnes",
    author_email="zxdavb@gmail.com",
    url=URL,
    download_url=f"{URL}/archive/{VERSION}.tar.gz",
    install_requires=[val.strip() for val in open("requirements.txt")],
    long_description=LONG_DESCRIPTION,
    long_description_content_type="text/markdown",
    packages=find_packages(exclude=["test", "docs"]),
    version=VERSION,
    license="Apache 2",
    python_requires=">=3.7",
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "License :: OSI Approved :: Apache Software License",
        "Operating System :: OS Independent",
        "Programming Language :: Python :: 3.7",
        "Topic :: Home Automation",
    ],
    cmdclass={
        "verify": VerifyVersionCommand,
    },
)
